//	This file shows you how to use key_states to move a player in a simple world.

//	First we define the ATTACK key as a power of 2.
#define ATTACK 16
/*
	Check the file 'demo.dmf' to see how the macros have been defined to use this
	ATTACK key, as well as the standard movement keys.
	*/


/*
	Now we define a global object to take care of our game loop. This object holds
	a list of players, and every 0.4/10 of a second will loop through all players
	and allow them to do whatever they want, such as move around or attack. Note
	how when a mob logs in, we add it to the game and reset its key_state, using
	client.reset_keys().
	*/
var/game/game = new()
game
	var
		speed = 0.4
		list/players = new()
	New()
		loop()
		world.tick_lag = speed
	proc/loop()
		for(var/mob/player in players)
			player.behavior()
		spawn(speed)
			loop()
mob
	Login()
		. = ..()
		game.players.Add(src)
		client.reset_keys()


/*
	Now we redefine mobs so that they use a very simple form of "pixel movement".
	Don't worry too much about this portion. Just note how we're using the variables
	_x and _y to position the mob, and the proc redraw() translates those coordinates
	into map coordinates and pixel offsets. Again, don't worry about this block.
	*/
mob
	var
		_x = 96
		_y = 96
		speed = 1
	proc/redraw()
		x = round((_x-_x%world.icon_size)/world.icon_size)+1
		y = round((_y-_y%world.icon_size)/world.icon_size)+1
		pixel_x = _x%world.icon_size
		pixel_y = _y%world.icon_size


/*
	This is the important part. This proc, behavior(), is called by the game object to
	give the player a chance to move around and attack. Note the use of key_press(),
	key_state(), and clear_press().
	*/
mob/proc/behavior()
	// Check client's key_press, and then clear it.
	if(client.key_press(ATTACK))
		world << "<b>Attack!</b> Here's where you'd make your character attack, or do whatever."
		client.clear_press(~ATTACK)
	// Check client's key_state and move character.
	if(client.key_state(NORTH))
		_y += speed
		dir = NORTH
	if(client.key_state(SOUTH)){ _y -= speed; dir = SOUTH}
	if(client.key_state(EAST )){ _x += speed; dir = EAST }
	if(client.key_state(WEST )){ _x -= speed; dir = WEST }
	// Make sure player is within map's range. Don't worry about these lines.
	_x = max(world.icon_size, min((world.maxx-2)*world.icon_size, _x))
	_y = max(world.icon_size, min((world.maxy-2)*world.icon_size, _y))
	// Reposition mob on map, to match it's _x and _y values.
	redraw()




// The rest of this file defines a simple testing world.
world
	maxx = 11
	maxy = 11
	maxz = 1
	icon_size = 16
	New()
		. = ..()
		for(var/y_pos = 1 to maxy)
			for(var/x_pos = 1 to maxx)
				if(y_pos == maxy || y_pos == 1 || x_pos == maxx || x_pos == 1)
					new /turf/tree(locate(x_pos, y_pos, 1))
				else
					new /turf/grass(locate(x_pos, y_pos, 1))
		for(var/turf/T)
			world << "\icon[T]"
atom/icon = 'demo.dmi'
turf
	grass
		icon_state = "grass"
	tree
		icon_state = "tree"
		density = TRUE
mob
	icon_state = "stand"
	animate_movement = FALSE